@@ -1,14 +1,24 @@
+import logging
 import unittest
 
 import responses
 from mock import MagicMock
-from syncloud.insider.dns import Dns
+from syncloud.app import logger
+from syncloud.insider.dns import Dns, ip_address
 from syncloud.insider.port_mapper import PortMapper
 from convertible import reformat
 
 from syncloud.insider.config import Port, Domain, Service
 from helpers import get_port_config, get_domain_config, get_service_config
 
+from syncloud.tools import config
+from syncloud.tools import footprint
+from syncloud.tools import id
+
+import convertible
+
+logger.init(level=logging.DEBUG, console=True)
+
 
 def get_upnpc(external_ip):
     upnpc = MagicMock()
@@ -50,19 +60,62 @@ class TestDns(unittest.TestCase):
         dns = Dns('http://api.domain.com', 'domain.com', domain_config, service_config, port_mapper)
         dns.sync()
 
+        local_ip_address = ip_address()
+
         expected_request = '''
 {
     "services": [
-        {"name": "ownCloud", "protocol": "http", "type": "_http._tcp", "port": 10000, "url": "owncloud"},
-        {"name": "SSH", "protocol": "https", "type": "_http._tcp", "port": 10001, "url": null}
+        {"name": "ownCloud", "protocol": "http", "type": "_http._tcp", "port": 10000, "local_port": 80, "url": "owncloud"},
+        {"name": "SSH", "protocol": "https", "type": "_http._tcp", "port": 10001, "local_port": 81, "url": null}
     ],
     "ip": "192.167.44.52",
+    "local_ip": "#local_ip",
     "token": "some_update_token"
 }
 '''
+        expected_request = expected_request.replace('#local_ip', local_ip_address)
 
         self.assertSingleRequest(expected_request)
 
+    @responses.activate
+    def test_sync_server_side_client_ip(self):
+        service_config = get_service_config([
+            Service("ownCloud", "http", "_http._tcp", 80, url="owncloud"),
+            Service("SSH", "https", "_http._tcp", 81, url=None)
+        ])
+        port_config = get_port_config([Port(80, 10000), Port(81, 10001)])
+        upnpc = get_upnpc(external_ip='10.1.1.1')
+        upnpc.mapped_ports = MagicMock(side_effect=[[], [10000]])
+
+        port_mapper = get_port_mapper(port_config, upnpc)
+        domain_config = get_domain_config(Domain('boris', 'some_update_token'))
+
+        responses.add(responses.POST,
+                      "http://api.domain.com/domain/update",
+                      status=200,
+                      body="{'message': 'Domain was updated'}",
+                      content_type="application/json")
+
+        dns = Dns('http://api.domain.com', 'domain.com', domain_config, service_config, port_mapper)
+        dns.sync()
+
+        local_ip_address = ip_address()
+
+        expected_request = '''
+{
+    "services": [
+        {"name": "ownCloud", "protocol": "http", "type": "_http._tcp", "port": 10000, "local_port": 80, "url": "owncloud"},
+        {"name": "SSH", "protocol": "https", "type": "_http._tcp", "port": 10001, "local_port": 81, "url": null}
+    ],
+    "token": "some_update_token",
+    "local_ip": "#local_ip"
+}
+'''
+        expected_request = expected_request.replace('#local_ip', local_ip_address)
+
+        self.assertSingleRequest(expected_request)
+
+
     @responses.activate
     def test_sync_server_error(self):
         service_config = get_service_config([Service("ownCloud", "http", "_http._tcp", 80, url="owncloud")])
@@ -86,6 +139,10 @@ class TestDns(unittest.TestCase):
 
     @responses.activate
     def test_link_success(self):
+        config.footprints['my-PC'] = footprint.footprint()
+        config.titles['my-PC'] = 'My PC'
+        device_id = id.id()
+
         domain_config = get_domain_config(None)
 
         responses.add(responses.POST,
@@ -101,8 +158,15 @@ class TestDns(unittest.TestCase):
         self.assertEquals(result.user_domain, "boris")
         self.assertEquals(result.update_token, "some_update_token")
 
-        # expected_request = '{"password": "pass1234", "email": "boris@mail.com", "user_domain": "boris"}'
-        # self.assertSingleRequest(expected_request)
+        expected_request_data = {
+            "password": "pass1234",
+            "email": "boris@mail.com",
+            "user_domain": "boris",
+            'device_mac_address': device_id.mac_address,
+            'device_name': device_id.name,
+            'device_title': device_id.title,
+        }
+        self.assertSingleRequest(convertible.to_json(expected_request_data))
 
         domain = domain_config.load()
         self.assertIsNotNone(domain)
@@ -111,6 +175,9 @@ class TestDns(unittest.TestCase):
 
     @responses.activate
     def test_link_server_error(self):
+        config.footprints['my-PC'] = footprint.footprint()
+        config.titles['my-PC'] = 'My PC'
+
         domain_config = get_domain_config(None)
 
         responses.add(responses.POST,
